/**
 * A lightweight image loader plugin for Vue.js
 *
 * @version 3.0.1
 * @author Charlie LEDUC <contact@graphique.io>
 * @license ISC
 * @requires 'vue'
 */

interface Callback {
  (completed: boolean, n: number): void
}

function _load(resource: string, callback: (image: HTMLImageElement) => void): void {
  var _img = new Image()
  _img.onload = () => {
    callback(_img)
  }
  _img.src = resource
}

function _set(source: HTMLImageElement, target: HTMLImageElement): void {
  target.src = source.src
  target.width = source.width
  target.height = source.height

  var list = ['img-square', 'img-portrait', 'img-landscape']
  list.forEach(c => {
    target.classList.remove(c)
  })

  var aspectClass = 'img-square'
  if (source.width < source.height) {
    aspectClass = 'img-portrait'
  } else {
    if (source.width / source.height >= 10 / 9) {
      aspectClass = 'img-landscape'
    }
  }
  if (!target.classList.contains(aspectClass)) {
    target.classList.add(aspectClass)
  }

  target.removeAttribute('data-src')
}

function _directiveFn(el: HTMLImageElement, binding: any): void {
  var resource = binding.value
  if (resource === el.src) {
    return
  }

  el.src = 'data:image/gif;base64,R0lGODlhAQABAIAAAAAAAP///yH5BAEAAAAALAAAAAABAAEAAAIBRAA7'
  el.setAttribute('data-src', resource)

  _load(resource, img => {
    _set(img, el)
  })
}

function preload(images: string[], callback: Callback) {
  var l = images.length
  var n = 0
  var completed = false
  if (images.length) {
    for (let i = 0; i < images.length; i++) {
      var resource = images[i]
      _load(resource, (img) => {
        n++
        if (typeof callback === 'function') {
          callback(completed, n / l)
          if (n >= l) {
            completed = true
            callback(completed, 1)
          }
        }
      })
    }
  } else {
    if (typeof callback === 'function') {
      completed = true
      callback(completed, 1)
    }
  }
}

const plugin = {
  install: (app: any, ...options: any[]) => {
    app.directive('onload', {
      mounted(el: any, binding: any, vnode: any) {
        if (vnode.type !== 'img') {
          console.warn('Current node is not an image!')
          return
        }
        _directiveFn(el, binding)
      },
      updated(el: any, binding: any, vnode: any) {
        if (vnode.type !== 'img') {
          console.warn('Current node is not an image!')
          return
        }
        _directiveFn(el, binding)
      }
    })
  }
}

export {
  preload,
  plugin
}
